import random
from unittest import TestCase

from core.constants import CellValue, CellValueRanges
from core.logic import Logic
from core.state import World, Layer


class TestWorld(TestCase):

    def test_setget(self):
        world = World((14, 7))
        self.assertEqual(14, world.width)
        self.assertEqual(7, world.height)

        ground = world.ground
        for y in range(world.height):
            for x in range(world.width):
                self.assertEqual(CellValue.GROUND_SEA, ground[x, y])

        ground[3, 4] = CellValue.GROUND_EARTH
        for y in range(world.height):
            for x in range(world.width):
                if x == 3 and y == 4:
                    self.assertEqual(CellValue.GROUND_EARTH, ground[x, y])
                else:
                    self.assertEqual(CellValue.GROUND_SEA, ground[x, y])

    def test_logic(self):
        world = World((13, 11))
        logic = Logic(world)
        ground = world.ground
        ground.fill(CellValue.GROUND_EARTH)
        for layerName in world.layerNames:
            if layerName == "ground":
                continue
            coords = (
                random.randint(0, world.width - 1),
                random.randint(0, world.height - 2)
            )
            valueRange = CellValueRanges[layerName]
            minValue = valueRange[0]
            maxValue = valueRange[1]
            value = CellValue(random.randint(minValue, maxValue - 1))
            Command = logic.getSetLayerValueCommand(layerName)

            command = Command(coords, value)
            logic.addCommand(command)
            logic.executeCommands()
            layer = world[layerName]
            self.assertEqual(value, layer[coords])

            command = Command(coords, CellValue.NONE)
            logic.addCommand(command)
            logic.executeCommands()
            layer = world[layerName]
            self.assertEqual(CellValue.NONE, layer[coords])

