from pygame import Surface

from core.constants import CellValue
from core.state import World
from tools.tilecodes import mask8, code8
from .LayerComponent import LayerComponent
from ...theme.Theme import Theme


class GroundComponent(LayerComponent):
    def __init__(self, theme: Theme, world: World):
        super().__init__(theme, world, "ground")
        self.__code2rect = self.tileset.getCode8Rects(0, 0)

    def render(self, surface: Surface):
        super().render(surface)
        tileset = self.tileset.surface
        tilesRects = self.tileset.getTilesRects()
        tileWidth, tileHeight = self.tileset.tileSize
        for y in range(self.layer.height):
            for x in range(self.layer.width):
                value = self.layer[x, y]
                if value == CellValue.NONE:
                    continue
                tile = (x * tileWidth, y * tileHeight)
                if self.autoTiling:
                    rects = tilesRects[value]
                    tileCount = len(rects)
                    rectIndex = self.noise[y][x] % tileCount
                    surface.blit(tileset, tile, rects[rectIndex])
                    if value == CellValue.GROUND_SEA:
                        neighbors = self.layer.getNeighbors8((x, y))
                        mask = mask8(neighbors, CellValue.GROUND_SEA)
                        code = code8(mask)
                        rect = self.__code2rect[code]
                        surface.blit(tileset, tile, rect)
                else:
                    rects = tilesRects[value]
                    surface.blit(tileset, tile, rects[0])
