from pygame import Surface

from core.constants import CellValue, directions, Direction
from core.state import World
from tools.tilecodes import mask4, combine4, code4
from .LayerComponent import LayerComponent
from ...theme.Theme import Theme


class ImpassableComponent(LayerComponent):
    def __init__(self, theme: Theme, world: World):
        super().__init__(theme, world, "impassable")
        self.__ground = world.getLayer("ground")
        self.__river_code2rect = self.tileset.getCode4Rects(0, 1)
        self.__riverMouth = {
            Direction.LEFT: self.tileset.getTileRect("riverMouthLeft"),
            Direction.RIGHT: self.tileset.getTileRect("riverMouthRight"),
            Direction.TOP: self.tileset.getTileRect("riverMouthTop"),
            Direction.BOTTOM: self.tileset.getTileRect("riverMouthBottom"),
        }

    def render(self, surface: Surface):
        super().render(surface)
        tileset = self.tileset.surface
        tilesRects = self.tileset.getTilesRects()
        tileWidth, tileHeight = self.tileset.tileSize
        for y in range(self.layer.height):
            for x in range(self.layer.width):
                value = self.layer[x, y]
                tile = (x * tileWidth, y * tileHeight)
                if self.autoTiling:
                    if value == CellValue.NONE:
                        groundValue = self.__ground.getValue((x, y))
                        if groundValue == CellValue.GROUND_SEA:
                            for direction in directions:
                                if self.layer.getValue((x, y), direction) == CellValue.IMPASSABLE_RIVER:
                                    rect = self.__riverMouth[direction]
                                    surface.blit(tileset, tile, rect)
                    elif value == CellValue.IMPASSABLE_RIVER:
                        neighbors = self.layer.getNeighbors4((x, y))
                        mask = mask4(neighbors, CellValue.IMPASSABLE_RIVER)
                        mask = combine4(mask, mask4(neighbors, CellValue.IMPASSABLE_MOUNTAIN))
                        neighbors = self.__ground.getNeighbors4((x, y))
                        mask = combine4(mask, mask4(neighbors, CellValue.GROUND_SEA))
                        code = code4(mask)
                        rect = self.__river_code2rect[code]
                        surface.blit(tileset, tile, rect)
                    else:
                        rects = tilesRects[value]
                        tileCount = len(rects)
                        rectIndex = self.noise[y][x] % tileCount
                        rect = rects[rectIndex]
                        surface.blit(tileset, tile, rect)
                elif value != CellValue.NONE:
                    rects = tilesRects[value]
                    surface.blit(tileset, tile, rects[0])

