from __future__ import annotations

import random
from typing import Tuple, List

from pygame import Surface

from core.state import World, ILayerListener, Layer
from ..Component import Component
from ...theme.Theme import Theme
from ...theme.Tileset import Tileset


class LayerComponent(Component, ILayerListener):

    def __init__(self, theme: Theme, world: World, name: str):
        super().__init__(theme)
        self.__layer = world.getLayer(name)
        self.__tileset = theme.getTileset(name)
        self.__needRefresh = True
        self.__autoTiling = True

        # Noise for tile generation
        self.__noise = []
        random.seed(name)
        width, height = self.__layer.size
        for y in range(height):
            row = []
            for x in range(width):
                row.append(random.randint(0, 100000))
            self.__noise.append(row)

    @property
    def layer(self) -> Layer:
        return self.__layer

    @property
    def tileset(self) -> Tileset:
        return self.__tileset

    @property
    def noise(self) -> List[List[int]]:
        return self.__noise

    @property
    def autoTiling(self) -> bool:
        return self.__autoTiling

    @autoTiling.setter
    def autoTiling(self, b: bool):
        self.__autoTiling = b
        self.__needRefresh = True

    # Component

    def needRefresh(self) -> bool:
        return self.__needRefresh

    def render(self, surface: Surface):
        self.__needRefresh = False

    # Layer listener

    def cellChanged(self, layer: Layer, cell: Tuple[int, int]):
        self.__needRefresh = True
