from core.state import World
from .GroundComponent import GroundComponent
from .ImpassableComponent import ImpassableComponent
from .LayerComponent import LayerComponent
from .ObjectsComponent import ObjectsComponent
from ...theme.Theme import Theme


class LayerComponentFactory:
    def __init__(self, theme: Theme, world: World):
        self.__name2layer = {
            "ground": lambda: GroundComponent(theme, world),
            "impassable": lambda: ImpassableComponent(theme, world),
            "objects": lambda: ObjectsComponent(theme, world)
        }

    def create(self, name: str) -> LayerComponent:
        if name not in self.__name2layer:
            raise ValueError(f"Invalid layer '{name}'")
        return self.__name2layer[name]()
