from pygame import Surface

from core.constants import CellValue
from core.state import World
from tools.tilecodes import mask4, code4, combine4
from .LayerComponent import LayerComponent
from ...theme.Theme import Theme


class ObjectsComponent(LayerComponent):
    def __init__(self, theme: Theme, world: World):
        super().__init__(theme, world, "objects")
        self.__impassableLayer = world.getLayer("impassable")
        self.__bridgeDirtRects = self.tileset.getTileRects("bridgeDirt")
        self.__bridgeStoneRects = self.tileset.getTileRects("bridgeStone")
        self.__roadDirt_code2rect = self.tileset.getCode4Rects(0, 3)
        self.__roadStone_code2rect = self.tileset.getCode4Rects(4, 3)

    def render(self, surface: Surface):
        super().render(surface)
        tileset = self.tileset.surface
        tilesRects = self.tileset.getTilesRects()
        tileWidth, tileHeight = self.tileset.tileSize
        for y in range(self.layer.height):
            for x in range(self.layer.width):
                value = self.layer[x, y]
                if value == CellValue.NONE:
                    continue
                tile = (x * tileWidth, y * tileHeight)
                if self.autoTiling:
                    if value in [CellValue.OBJECTS_ROAD_DIRT, CellValue.OBJECTS_ROAD_STONE]:
                        # Bridge case
                        impassableValue = self.__impassableLayer.getValue((x, y))
                        if impassableValue == CellValue.IMPASSABLE_RIVER:
                            left = self.__impassableLayer.getValue((x - 1, y))
                            right = self.__impassableLayer.getValue((x + 1, y))
                            riverHorizontal = (left == CellValue.IMPASSABLE_RIVER) and (right == CellValue.IMPASSABLE_RIVER)
                            if value == CellValue.OBJECTS_ROAD_DIRT:
                                rect = self.__bridgeDirtRects[riverHorizontal]
                            else:
                                rect = self.__bridgeStoneRects[riverHorizontal]
                        else:
                            neighbors = self.layer.getNeighbors4((x, y))
                            mask = mask4(neighbors, CellValue.OBJECTS_ROAD_DIRT)
                            mask = combine4(mask, mask4(neighbors, CellValue.OBJECTS_ROAD_STONE))
                            code = code4(mask)
                            rect = self.__roadDirt_code2rect[code]
                    else:
                        rects = tilesRects[value]
                        tileCount = len(rects)
                        rectIndex = self.noise[y][x] % tileCount
                        rect = rects[rectIndex]
                    surface.blit(tileset, tile, rect)
                else:
                    rect = tilesRects[value][0]
                    surface.blit(tileset, tile, rect)

